using System;
using System.Data;
using System.Collections;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.IO;

using CMS.FormEngine;
using CMS.FormControls;
using CMS.WorkflowEngine;
using CMS.GlobalHelper;
using CMS.CMSHelper;
using CMS.TreeEngine;
using CMS.SettingsProvider;
using CMS.SiteProvider;
using CMS.ExtendedControls;
using CMS.PortalEngine;
using CMS.EventLog;
using CMS.DataEngine;
using CMS.URLRewritingEngine;
using CMS.Controls;
using CMS.UIControls;
using CMS.LicenseProvider;

using TreeNode = CMS.TreeEngine.TreeNode;


public partial class CMSAdminControls_UserContributions_EditForm : CMSUserControl
{
    #region "Variables"

    /// <summary>
    /// On after aprove event
    /// </summary>
    public event EventHandler OnAfterApprove = null;

    /// <summary>
    /// On after reject event
    /// </summary>
    public event EventHandler OnAfterReject = null;


    /// <summary>
    /// On after delete event
    /// </summary>
    public event EventHandler OnAfterDelete = null;


    private bool confirmChanges = false;
    private TreeNode node = null;
    private TreeProvider tree = null;
    private DataClassInfo ci = null;

    /// <summary>
    /// Data properties variable
    /// </summary>
    private CMSDataProperties mDataProperties = new CMSDataProperties();

    #endregion


    #region "Document properties"

    /// <summary>
    /// Culture code
    /// </summary>
    public string CultureCode
    {
        get
        {
            return this.mDataProperties.CultureCode;
        }
        set
        {
            this.mDataProperties.CultureCode = value;
        }
    }


    /// <summary>
    /// Site name
    /// </summary>
    public string SiteName
    {
        get
        {
            return this.mDataProperties.SiteName;
        }
        set
        {
            this.mDataProperties.SiteName = value;
        }
    }

    #endregion


    #region "Public properties"

    /// <summary>
    /// Returns true if new document mode
    /// </summary>
    public bool NewDocument
    {
        get
        {
            return (this.Action.ToLower() == "new");
        }
    }


    /// <summary>
    /// Returns true if new culture mode
    /// </summary>
    public bool NewCulture
    {
        get
        {
            return (this.Action.ToLower() == "newculture");
        }
    }


    /// <summary>
    /// Returns true in delete mode
    /// </summary>
    public bool Delete
    {
        get
        {
            return (this.Action.ToLower() == "delete");
        }
    }


    /// <summary>
    /// Node ID
    /// </summary>
    public int NodeID
    {
        get
        {
            return ValidationHelper.GetInteger(ViewState["NodeID"], 0);
        }
        set
        {
            ViewState["NodeID"] = value;
            this.menuElem.NodeID = value;
        }
    }


    /// <summary>
    /// Form Action (mode)
    /// </summary>
    public string Action
    {
        get
        {
            return ValidationHelper.GetString(ViewState["Action"], "");
        }
        set
        {
            ViewState["Action"] = value;
        }
    }


    /// <summary>
    /// Class ID
    /// </summary>
    public int ClassID
    {
        get
        {
            return ValidationHelper.GetInteger(ViewState["ClassID"], 0);
        }
        set
        {
            ViewState["ClassID"] = value;
        }
    }


    /// <summary>
    /// Alternative form name
    /// </summary>
    public string AlternativeFormName
    {
        get
        {
            return ValidationHelper.GetString(ViewState["AlternativeFormName"], null);
        }
        set
        {
            ViewState["AlternativeFormName"] = value;
        }
    }


    /// <summary>
    /// Form validation error message
    /// </summary>
    public string ValidationErrorMessage
    {
        get
        {
            return ValidationHelper.GetString(ViewState["ValidationErrorMessage"], null);
        }
        set
        {
            ViewState["ValidationErrorMessage"] = value;
        }
    }


    /// <summary>
    /// Template ID
    /// </summary>
    public int TemplateID
    {
        get
        {
            return ValidationHelper.GetInteger(ViewState["TemplateID"], 0);
        }
        set
        {
            ViewState["TemplateID"] = value;
        }
    }


    /// <summary>
    /// Owner ID
    /// </summary>
    public int OwnerID
    {
        get
        {
            return ValidationHelper.GetInteger(ViewState["OwnerID"], 0);
        }
        set
        {
            ViewState["OwnerID"] = value;
        }
    }


    /// <summary>
    /// New item page template code name
    /// </summary>
    public string NewItemPageTemplate
    {
        get
        {
            return ValidationHelper.GetString(ViewState["NewItemPageTemplate"], "");
        }
        set
        {
            ViewState["NewItemPageTemplate"] = value;

            // Get template and set template ID
            PageTemplateInfo pti = PageTemplateInfoProvider.GetPageTemplateInfo(value);
            if (pti != null)
            {
                this.TemplateID = pti.PageTemplateId;
            }
        }
    }


    /// <summary>
    /// Returns true if the changes should be saved
    /// </summary>
    public bool SaveChanges
    {
        get
        {
            return ValidationHelper.GetBoolean(HttpContext.Current.Request.Params["saveChanges"], false);
        }
    }


    /// <summary>
    /// List of allowed child classes separated by semicolon
    /// </summary>
    public string AllowedChildClasses
    {
        get
        {
            return ValidationHelper.GetString(ViewState["AllowedChildClasses"], "");
        }
        set
        {
            ViewState["AllowedChildClasses"] = value;
        }
    }


    /// <summary>
    /// Document ID to use for default data of new culture version
    /// </summary>
    public int CopyDefaultDataFromDocumentID
    {
        get
        {
            return ValidationHelper.GetInteger(ViewState["CopyDefaultDataFromDocumentID"], 0);
        }
        set
        {
            ViewState["CopyDefaultDataFromDocumentID"] = value;
        }
    }


    /// <summary>
    /// If true, form allows deleting the document
    /// </summary>
    public bool AllowDelete
    {
        get
        {
            return ValidationHelper.GetBoolean(ViewState["AllowDelete"], true);
        }
        set
        {
            ViewState["AllowDelete"] = value;
        }
    }


    /// <summary>
    /// Gets or sets the value that indicates whether document permissions are checked
    /// </summary>
    public bool CheckPermissions
    {
        get
        {
            return ValidationHelper.GetBoolean(ViewState["CheckPermissions"], false);
        }
        set
        {
            ViewState["CheckPermissions"] = value;
            this.menuElem.CheckPermissions = value;
        }
    }


    /// <summary>
    /// Prefix of the action functions
    /// </summary>
    public string FunctionsPrefix
    {
        get
        {
            return ValidationHelper.GetString(ViewState["FunctionsPrefix"], "");
        }
        set
        {
            ViewState["FunctionsPrefix"] = value;
            this.menuElem.FunctionsPrefix = value;
        }
    }


    /// <summary>
    /// Editing form
    /// </summary>
    public CMSForm CMSForm
    {
        get
        {
            return formElem;
        }
    }

    #endregion


    #region "Methods"

    protected override void CreateChildControls()
    {
        LicenseHelper.CheckFeatureAndRedirect(UrlHelper.GetCurrentDomain(), FeatureEnum.UserContributions);

        this.Page.ClientScript.RegisterClientScriptBlock(typeof(string), ScriptHelper.DIALOG_SCRIPT_KEY, ScriptHelper.DialogScript);

        confirmChanges = SettingsKeyProvider.GetBoolValue(this.SiteName + ".CMSConfirmChanges");

        tree = new TreeProvider(CMSContext.CurrentUser);
        formElem.TreeProvider = tree;

        ReloadData(false);
    }


    /// <summary>
    /// Reloads control.
    /// </summary>
    /// <param name="forceReload">Forces nested CMSForm reload if true</param>
    public void ReloadData(bool forceReload)
    {
        this.titleElem.TitleImage = "";
        this.titleElem.TitleText = "";

        if (!confirmChanges)
        {
            this.ltlScript.Text += ScriptHelper.GetScript("confirmChanges = false;");
        }

        // Get the node
        if (NewCulture || NewDocument)
        {
            node = DocumentHelper.GetDocument(NodeID, TreeProvider.ALL_CULTURES, tree);
            if (NewCulture)
            {
                DocumentHelper.ClearWorkflowInformation(node);
            }
        }
        else
        {
            node = DocumentHelper.GetDocument(NodeID, this.CultureCode, tree);
        }

        this.pnlSelectClass.Visible = false;
        this.pnlEdit.Visible = false;
        this.pnlInfo.Visible = false;
        this.pnlNewCulture.Visible = false;
        this.pnlDelete.Visible = false;

        // If node found, init the form
        if (node != null)
        {
            if (Delete)
            {
                // Delete document
                this.pnlDelete.Visible = true;

                this.titleElem.TitleText = ResHelper.GetString("Content.DeleteTitle");
                this.titleElem.TitleImage = "~/App_Themes/Default/Images/CMSDesk/Content/delete.gif";

                this.chkAllCultures.Text = ResHelper.GetString("ContentDelete.AllCultures");
                this.chkDestroy.Text = ResHelper.GetString("ContentDelete.Destroy");

                this.lblQuestion.Text = ResHelper.GetString("ContentDelete.Question");
                this.btnYes.Text = ResHelper.GetString("general.yes");
                this.btnNo.Text = ResHelper.GetString("general.no");

                DataSet culturesDS = CultureInfoProvider.GetSiteCultures(this.SiteName);
                if ((DataHelper.DataSourceIsEmpty(culturesDS)) || (culturesDS.Tables[0].Rows.Count <= 1))
                {
                    this.chkAllCultures.Visible = false;
                    this.chkAllCultures.Checked = true;
                }

                if (node.IsLink)
                {
                    this.titleElem.TitleText = ResHelper.GetString("Content.DeleteTitleLink") + " \"" + HTMLHelper.HTMLEncode(node.NodeName) + "\"";
                    this.lblQuestion.Text = ResHelper.GetString("ContentDelete.QuestionLink");
                    this.chkAllCultures.Checked = true;
                    this.plcCheck.Visible = false;
                }
                else
                {
                    this.titleElem.TitleText = ResHelper.GetString("Content.DeleteTitle") + " \"" + HTMLHelper.HTMLEncode(node.NodeName) + "\"";
                }
            }
            else
            {
                if (NewDocument)
                {
                    this.titleElem.TitleImage = "~/App_Themes/Default/Images/CMSDesk/Content/new.gif";
                    this.titleElem.TitleText = ResHelper.GetString("Content.NewTitle");
                }

                if (NewDocument && (ClassID <= 0))
                {
                    // Select document type
                    this.pnlSelectClass.Visible = true;

                    // Get the allowed child classes
                    DataSet ds = DataClassInfoProvider.GetAllowedChildClasses(ValidationHelper.GetInteger(node.GetValue("NodeClassID"), 0), ValidationHelper.GetInteger(SiteInfoProvider.GetSiteInfo(this.SiteName).SiteID, 0));

                    ArrayList deleteRows = new ArrayList();

                    if (!DataHelper.DataSourceIsEmpty(ds))
                    {
                        // Get the unwanted classes
                        string allowed = this.AllowedChildClasses.Trim().ToLower();
                        if (allowed != "")
                        {
                            allowed = ";" + allowed + ";";
                        }

                        foreach (DataRow dr in ds.Tables[0].Rows)
                        {
                            string className = ValidationHelper.GetString(DataHelper.GetDataRowValue(dr, "ClassName"), "").ToLower();
                            // Document type is not allowed or user hasn't got permission, remove it from the data set
                            if (((allowed != "") && (!allowed.Contains(";" + className + ";"))) ||
                                (this.CheckPermissions && (!CMSContext.CurrentUser.IsAuthorizedPerResource("CMS.Content", "Create") && !CMSContext.CurrentUser.IsAuthorizedPerClassName(className, "Create"))))
                            {
                                deleteRows.Add(dr);
                            }
                        }

                        // Remove the rows
                        foreach (DataRow dr in deleteRows)
                        {
                            ds.Tables[0].Rows.Remove(dr);
                        }
                    }

                    // Check if some classes are available
                    if (!DataHelper.DataSourceIsEmpty(ds))
                    {
                        // If number of classes is more than 1 display them in grid
                        if (ds.Tables[0].Rows.Count > 1)
                        {
                            ds.Tables[0].DefaultView.Sort = "ClassDisplayName";
                            lblError.Visible = false;
                            lblInfo.Visible = true;
                        }
                        // else show form of the only class
                        else
                        {
                            this.ClassID = ValidationHelper.GetInteger(DataHelper.GetDataRowValue(ds.Tables[0].Rows[0], "ClassID"), 0);
                            this.ReloadData(true);
                            return;
                        }
                    }
                    else
                    {
                        lblError.Visible = true;
                        lblError.Text = ResHelper.GetString("Content.NoAllowedChildDocuments");
                        lblInfo.Visible = false;
                    }

                    this.gridClasses.DataSource = ds.Tables[0].DefaultView.Table;
                    this.gridClasses.DataBind();

                    this.lblInfo.Text = ResHelper.GetString("Content.NewInfo");
                }
                else
                {
                    // Display the form
                    this.pnlEdit.Visible = true;

                    // Try to get GroupID if group context exists
                    int currentGroupId = ModuleCommands.CommunityGetCurrentGroupID();

                    this.ltlScript.Text += ScriptHelper.GetScript(
                        "function PassiveRefresh(nodeId, selectNodeId) {} \n" +
                        "function " + this.FunctionsPrefix + "SaveDocument(NodeID) { AllowSubmit(); " + this.Page.ClientScript.GetPostBackEventReference(this.btnSave, null) + "; } \n" +
                        "function " + this.FunctionsPrefix + "Approve(NodeID) { SubmitAction(); " + this.Page.ClientScript.GetPostBackEventReference(this.btnApprove, null) + "; } \n" +
                        "function " + this.FunctionsPrefix + "Reject(NodeID) { SubmitAction(); " + this.Page.ClientScript.GetPostBackEventReference(this.btnReject, null) + "; } \n" +
                        "function " + this.FunctionsPrefix + "CheckIn(NodeID) { " + this.Page.ClientScript.GetPostBackEventReference(this.btnCheckIn, null) + "; } \n" +
                        "var confirmSave='" + ResHelper.GetString("Content.ConfirmSave") + "'; var confirmLeave='" + ResHelper.GetString("Content.ConfirmLeave") + "'; \n" +
                        "function " + this.FunctionsPrefix + "CheckOut(NodeID) { " + this.Page.ClientScript.GetPostBackEventReference(this.btnCheckOut, null) + "; } \n" +
                        "function " + this.FunctionsPrefix + "UndoCheckOut(NodeID) { if(!confirm(" + ScriptHelper.GetString(ResHelper.GetString("EditMenu.UndoCheckOutConfirmation")) + ")) return false; " + this.Page.ClientScript.GetPostBackEventReference(this.btnUndoCheckOut, null) + "; } \n" +
                        "function " + this.FunctionsPrefix + "Delete(NodeID) { " + this.Page.ClientScript.GetPostBackEventReference(this.btnDelete, null) + "; } \n" +
                        ((!string.IsNullOrEmpty(this.FunctionsPrefix)) ? "function SaveDocument(NodeID) { " + this.FunctionsPrefix + "SaveDocument(NodeID); } \n function " + this.FunctionsPrefix + "SpellCheck() { SpellCheck(); } \n" : "") +
                        "var spellURL = '" + ResolveUrl("~/CMSAdminControls/SpellChecker/SpellCheck.aspx") + "'; \n"
                    );

                    this.btnApprove.Attributes.Add("style", "display: none;");
                    this.btnCheckIn.Attributes.Add("style", "display: none;");
                    this.btnCheckOut.Attributes.Add("style", "display: none;");
                    this.btnReject.Attributes.Add("style", "display: none;");
                    this.btnSave.Attributes.Add("style", "display: none;");
                    this.btnUndoCheckOut.Attributes.Add("style", "display: none;");
                    this.btnDelete.Attributes.Add("style", "display: none;");

                    // CMSForm initialization
                    formElem.NodeId = node.NodeID;
                    formElem.SiteName = this.SiteName;
                    formElem.CultureCode = this.CultureCode;
                    formElem.ValidationErrorMessage = this.ValidationErrorMessage;
                    formElem.IsLiveSite = this.IsLiveSite;
                    // Set group ID if group context exists
                    formElem.GroupID = currentGroupId;

                    // Set the form mode
                    if (NewDocument)
                    {
                        ci = DataClassInfoProvider.GetDataClass(ClassID);
                        if (ci == null)
                        {
                            throw new Exception("[CMSAdminControls/EditForm.aspx]: Class ID '" + ClassID + "' not found.");
                        }

                        titleElem.TitleText = ResHelper.GetString("Content.NewTitle") + ": " + ci.ClassDisplayName;

                        // Set default template ID
                        formElem.DefaultPageTemplateID = TemplateID > 0 ? TemplateID : ci.ClassDefaultPageTemplateID;

                        // Set document owner
                        formElem.OwnerID = OwnerID;
                        formElem.FormMode = FormModeEnum.Insert;
                        string newClassName = ci.ClassName;
                        string newFormName = newClassName + ".default";
                        if (!String.IsNullOrEmpty(this.AlternativeFormName))
                        {
                            // Set the alternative form full name
                            formElem.AlternativeFormFullName = GetAltFormFullName(ci.ClassName);
                        }
                        if (newFormName.ToLower() != formElem.FormName.ToLower())
                        {
                            formElem.FormName = newFormName;
                        }
                    }
                    else if (NewCulture)
                    {
                        formElem.FormMode = FormModeEnum.InsertNewCultureVersion;
                        // Default data document ID
                        formElem.CopyDefaultDataFromDocumentId = this.CopyDefaultDataFromDocumentID;

                        ci = DataClassInfoProvider.GetDataClass(node.NodeClassName);
                        formElem.FormName = node.NodeClassName + ".default";
                        if (!String.IsNullOrEmpty(this.AlternativeFormName))
                        {
                            // Set the alternative form full name
                            formElem.AlternativeFormFullName = GetAltFormFullName(ci.ClassName);
                        }
                    }
                    else
                    {
                        formElem.FormMode = FormModeEnum.Update;
                        ci = DataClassInfoProvider.GetDataClass(node.NodeClassName);
                        formElem.FormName = "";
                        if (!String.IsNullOrEmpty(this.AlternativeFormName))
                        {
                            // Set the alternative form full name
                            formElem.AlternativeFormFullName = GetAltFormFullName(ci.ClassName);
                        }

                        formElem.LoadForm();
                    }

                    formElem.Visible = true;

                    ReloadForm();
                }
            }
        }
        else
        {
            // Try to get any culture of the document
            node = DocumentHelper.GetDocument(this.NodeID, TreeProvider.ALL_CULTURES, tree);
            if (node != null)
            {
                // Offer a new culture creation
                this.pnlNewCulture.Visible = true;

                this.titleElem.TitleText = ResHelper.GetString("Content.NewCultureVersionTitle") + " (" + HTMLHelper.HTMLEncode(CMSContext.CurrentUser.PreferredCultureCode) + ")";
                this.titleElem.TitleImage = "~/App_Themes/Default/Images/CMSDesk/Content/new.gif";

                this.lblNewCultureInfo.Text = ResHelper.GetString("ContentNewCultureVersion.Info");
                this.radCopy.Text = ResHelper.GetString("ContentNewCultureVersion.Copy");
                this.radEmpty.Text = ResHelper.GetString("ContentNewCultureVersion.Empty");

                radCopy.Attributes.Add("onclick", "ShowSelection();");
                radEmpty.Attributes.Add("onclick", "ShowSelection()");

                this.ltlScript.Text += ScriptHelper.GetScript(
                    "function ShowSelection() { \n" +
                    "   if (document.getElementById('" + this.radCopy.ClientID + "').checked) { document.getElementById('divCultures').style.display = 'block'; } \n" +
                    "   else { document.getElementById('divCultures').style.display = 'none'; } \n" +
                    "} \n"
                );

                this.btnOK.Text = ResHelper.GetString("ContentNewCultureVersion.Create");

                // Load culture versions
                SiteInfo si = SiteInfoProvider.GetSiteInfo(node.NodeSiteID);
                if (si != null)
                {
                    lstCultures.Items.Clear();

                    DataSet nodes = tree.SelectNodes(si.SiteName, node.NodeAliasPath, TreeProvider.ALL_CULTURES, false, null, null, null, 1, false);
                    foreach (DataRow nodeCulture in nodes.Tables[0].Rows)
                    {
                        ListItem li = new ListItem();
                        li.Text = CultureInfoProvider.GetCultureInfo(nodeCulture["DocumentCulture"].ToString()).CultureName;
                        li.Value = nodeCulture["DocumentID"].ToString();
                        lstCultures.Items.Add(li);
                    }
                    if (lstCultures.Items.Count > 0)
                    {
                        lstCultures.SelectedIndex = 0;
                    }
                }
            }
            else
            {
                this.pnlInfo.Visible = true;
                this.lblFormInfo.Text = ResHelper.GetString("EditForm.DocumentNotFound");
                this.formElem.StopProcessing = true;
            }
        }
    }


    private void ReloadForm()
    {
        this.lblWorkflowInfo.Text = "";

        formElem.Enabled = true;

        if ((node != null) && !NewDocument && !NewCulture)
        {
            bool authorized = true;

            // Check the permissions
            if (this.CheckPermissions)
            {
                // Check read permissions
                if (CMSContext.CurrentUser.IsAuthorizedPerTreeNode(NodeID, NodePermissionsEnum.Read) == AuthorizationResultEnum.Denied)
                {
                    RedirectToAccessDenied(String.Format(ResHelper.GetString("cmsdesk.notauthorizedtoreaddocument"), node.NodeAliasPath));
                }
                // Check modify permissions
                else if (CMSContext.CurrentUser.IsAuthorizedPerTreeNode(NodeID, NodePermissionsEnum.Modify) == AuthorizationResultEnum.Denied)
                {
                    formElem.Enabled = false;
                    lblWorkflowInfo.Text = String.Format(ResHelper.GetString("cmsdesk.notauthorizedtoeditdocument"), node.NodeAliasPath);
                    authorized = false;
                }
            }

            // If authorized
            if (authorized)
            {
                // Setup the workflow information
                WorkflowManager wm = new WorkflowManager(tree);
                WorkflowInfo wi = wm.GetNodeWorkflow(node);
                if ((wi != null) && (!NewCulture))
                {
                    // Get current step info
                    WorkflowStepInfo si = wm.GetStepInfo(node);
                    if (si != null)
                    {
                        bool allowApproval = true;
                        bool canApprove = wm.CanUserApprove(node, CMSContext.CurrentUser);
                        if (!(canApprove || (si.StepName.ToLower() == "edit") || (si.StepName.ToLower() == "published") || (si.StepName.ToLower() == "archived")))
                        {
                            formElem.Enabled = false;
                        }

                        // Check-in, Check-out
                        if (VersionManager.UseCheckInCheckOut(this.SiteName))
                        {
                            if (node.GetValue("DocumentCheckedOutByUserID") == null)
                            {
                                // If not checked out, add the check-out information
                                if (canApprove || (si.StepName.ToLower() == "edit") || (si.StepName.ToLower() == "published") || (si.StepName.ToLower() == "archived"))
                                {
                                    this.lblWorkflowInfo.Text = ResHelper.GetString("EditContent.DocumentCheckedIn");
                                }
                                this.formElem.Enabled = NewCulture;
                            }
                            else
                            {
                                // If checked out by current user, add the check-in button
                                int checkedOutBy = ValidationHelper.GetInteger(node.GetValue("DocumentCheckedOutByUserID"), 0);
                                if (checkedOutBy == CMSContext.CurrentUser.UserID)
                                {
                                    // Document is checked out
                                    this.lblWorkflowInfo.Text = ResHelper.GetString("EditContent.DocumentCheckedOut");
                                }
                                else
                                {
                                    // Checked out by somebody else
                                    this.lblWorkflowInfo.Text = ResHelper.GetString("EditContent.DocumentCheckedOutByAnother");
                                    this.formElem.Enabled = NewCulture;
                                }
                                allowApproval = false;
                            }
                        }

                        // Document approval
                        if (allowApproval)
                        {
                            switch (si.StepName.ToLower())
                            {
                                case "edit":
                                case "published":
                                case "archived":
                                    break;

                                default:
                                    // If the user is authorized to perform the step, display the approve and reject buttons
                                    if (!canApprove)
                                    {
                                        this.lblWorkflowInfo.Text += " " + ResHelper.GetString("EditContent.NotAuthorizedToApprove");
                                    }
                                    break;
                            }
                            this.lblWorkflowInfo.Text += " " + String.Format(ResHelper.GetString("EditContent.CurrentStepInfo"), si.StepDisplayName);
                        }
                    }
                }
            }
        }

        this.pnlWorkflowInfo.Visible = (this.lblWorkflowInfo.Text != "");

        this.menuElem.ShowDelete = this.AllowDelete && (this.Action.ToLower() == "edit");
        this.menuElem.NodeID = this.NodeID;
        this.menuElem.Action = this.Action.ToLower();

        this.CheckPermissions = this.CheckPermissions;

        this.menuElem.ReloadMenu();
    }


    /// <summary>
    /// Adds the alert message to the output request window
    /// </summary>
    /// <param name="message">Message to display</param>
    private void AddAlert(string message)
    {
        this.ltlScript.Text += ScriptHelper.GetAlertScript(message);
    }


    /// <summary>
    /// Adds the script to the output request window
    /// </summary>
    /// <param name="script">Script to add</param>
    private void AddScript(string script)
    {
        this.ltlScript.Text += ScriptHelper.GetScript(script);
    }


    /// <summary>
    /// Save button click event handler.
    /// </summary>
    protected void btnSave_Click(object sender, EventArgs e)
    {
        SaveDocument();
    }


    /// <summary>
    /// Saves new or existing document.
    /// </summary>
    private void SaveDocument()
    {
        if (IsBannedIP())
        {
            return;
        }

        if (NodeID > 0)
        {
            // Validate the form first
            if (formElem.BasicForm.ValidateData())
            {
                bool createAnother = NewDocument && ValidationHelper.GetBoolean(Request.Params["hidAnother"], false);
                bool autoCheck = false;
                bool useWorkflow = false;
                VersionManager vm = null;
                if (!NewDocument && !NewCulture)
                {
                    autoCheck = !VersionManager.UseCheckInCheckOut(this.SiteName);
                    WorkflowManager workflowMan = new WorkflowManager(tree);

                    // Get the document
                    node = DocumentHelper.GetDocument(NodeID, this.CultureCode, tree);

                    //test if the document uses workflow
                    if (workflowMan.GetNodeWorkflow(node) != null)
                    {
                        useWorkflow = true;
                    }
                }

                try
                {
                    // If not using check-in/check-out, check out automatically
                    if (autoCheck & useWorkflow)
                    {
                        if (node != null)
                        {
                            // Check out
                            vm = new VersionManager(tree);
                            vm.CheckOut(node);
                        }
                    }

                    // Tree node is returned from CMSForm.Save method
                    if ((node != null) && !NewDocument && !NewCulture)
                    {
                        formElem.Save(node);
                    }
                    else
                    {
                        // Special treatment for blog post (ensure parent month)
                        if (NewDocument && (ci != null) && (ci.ClassName.ToLower() == "cms.blogpost"))
                        {
                            // Get parent node
                            node = tree.SelectSingleNode(NodeID, TreeProvider.ALL_CULTURES);
                            if ((node != null) && (node.NodeClassName.ToLower() != "cms.blogmonth"))
                            {
                                // Prepare the data
                                DateTime startDate = ValidationHelper.GetDateTime(formElem.BasicForm.GetFieldValue("BlogPostDate"), DateTimePicker.NOT_SELECTED);
                                string monthName = startDate.ToString("MMMM") + " " + startDate.Year.ToString();
                                startDate = new DateTime(startDate.Year, startDate.Month, 1);

                                // Check existing
                                DataSet existingDS = tree.SelectNodes(this.SiteName, node.NodeAliasPath + "/%", TreeProvider.ALL_CULTURES, true, "cms.blogmonth", "BlogMonthStartingDate = '" + TableManager.GetDateTimeString(startDate) + "'", null, -1, false);
                                if (DataHelper.DataSourceIsEmpty(existingDS))
                                {
                                    // Create the month node
                                    node = new TreeNode("CMS.BlogMonth", tree);
                                    node.DocumentCulture = this.CultureCode;

                                    // Insert the month
                                    node.SetValue("BlogMonthName", monthName);
                                    node.SetValue("BlogMonthStartingDate", startDate);
                                    node.DocumentName = monthName;
                                    DocumentHelper.InsertDocument(node, NodeID, tree);

                                    this.formElem.NodeId = node.NodeID;
                                }
                                else
                                {
                                    // Set existing
                                    this.formElem.NodeId = ValidationHelper.GetInteger(existingDS.Tables[0].Rows[0]["NodeID"], 0);
                                }
                            }
                        }
                        node = formElem.Save();

                        // Clear cache if current document is blogpost or blog
                        if ((ci.ClassName.ToLower() == "cms.blogpost") || (ci.ClassName.ToLower() == "cms.blog"))
                        {
                            // Clear cache
                            if (CMSControlsHelper.CurrentPageManager != null)
                            {
                                CMSControlsHelper.CurrentPageManager.ClearCache();
                            }
                        }

                        // Set the edit mode
                        if (node != null)
                        {
                            this.NodeID = node.NodeID;
                            this.Action = "edit";
                            this.ReloadData(true);
                        }
                    }

                    if (node != null)
                    {
                        // Check in the document
                        if (autoCheck & useWorkflow)
                        {
                            vm.CheckIn(node, null, null);
                        }
                    }

                    ReloadForm();
                }
                catch (Exception ex)
                {
                    AddAlert(ResHelper.GetString("ContentRequest.SaveFailed") + " : " + ex.Message);
                }
            }
        }
    }


    /// <summary>
    /// Approve button click event handler.
    /// </summary>
    protected void btnApprove_Click(object sender, EventArgs e)
    {
        if (IsBannedIP())
        {
            return;
        }

        if (node != null)
        {
            // Validate the form first
            if (formElem.BasicForm.ValidateData())
            {
                // Save the changes first
                bool autoCheck = !VersionManager.UseCheckInCheckOut(this.SiteName);
                if (autoCheck)
                {
                    if (this.SaveChanges)
                    {
                        SaveDocument();
                    }
                    else
                    {
                        this.formElem.BasicForm.LoadControlValues();
                    }
                }

                // Approve the document - Go to next workflow step
                WorkflowManager wm = new WorkflowManager(tree);

                // Get original step
                WorkflowStepInfo originalStep = wm.GetStepInfo(node);

                // Approve the document
                WorkflowStepInfo nextStep = wm.MoveToNextStep(node, "");
                if (nextStep != null)
                {
                    string nextStepName = nextStep.StepName.ToLower();
                    string originalStepName = originalStep.StepName.ToLower();

                    // Send workflow e-mails
                    if (SettingsKeyProvider.GetBoolValue(SiteName + ".CMSSendWorkflowEmails"))
                    {
                        if (nextStepName == "published")
                        {
                            // Publish e-mails
                            wm.SendWorkflowEmails(node, CMSContext.CurrentUser, originalStep, nextStep, WorkflowActionEnum.Published, "");
                        }
                        else
                        {
                            // Approve e-mails
                            wm.SendWorkflowEmails(node, CMSContext.CurrentUser, originalStep, nextStep, WorkflowActionEnum.Approved, "");
                        }
                    }
                    ltlScript.Text += ScriptHelper.GetScript("PassiveRefresh(" + NodeID + ", " + node.NodeParentID + ");");

                    // Ensure correct message is displayed
                    if (nextStepName == "published")
                    {
                        formElem.lblInfo.Text += " " + ResHelper.GetString("workflowstep.customtopublished");
                    }
                    else if (originalStepName == "edit" && nextStepName != "published")
                    {
                        formElem.lblInfo.Text += " " + ResHelper.GetString("workflowstep.edittocustom");
                    }
                    else if (originalStepName != "edit" && nextStepName != "published" && nextStepName != "archived")
                    {
                        formElem.lblInfo.Text += " " + ResHelper.GetString("workflowstep.customtocustom");
                    }
                    else
                    {
                        formElem.lblInfo.Text += " " + ResHelper.GetString("ContentEdit.WasApproved");
                    }
                }
                else
                {
                    // Workflow has been removed
                    AddScript("     SelectNode(" + NodeID + ");");
                    formElem.lblInfo.Text += " " + ResHelper.GetString("ContentEdit.WasApproved");
                }
            }

            ReloadForm();

            RaiseOnAfterApprove();
        }
    }


    /// <summary>
    /// Reject button click event handler.
    /// </summary>
    protected void btnReject_Click(object sender, EventArgs e)
    {
        if (IsBannedIP())
        {
            return;
        }

        // Reject the document - Go to previous workflow step
        if (node != null)
        {
            // Validate the form first
            if (formElem.BasicForm.ValidateData())
            {
                // Save the document first
                bool autoCheck = !VersionManager.UseCheckInCheckOut(this.SiteName);
                if (autoCheck)
                {
                    if (this.SaveChanges)
                    {
                        SaveDocument();
                    }
                    else
                    {
                        this.formElem.BasicForm.LoadControlValues();
                    }
                }

                WorkflowManager wm = new WorkflowManager(tree);

                // Get original step
                WorkflowStepInfo originalStep = wm.GetStepInfo(node);

                // Reject the document
                WorkflowStepInfo previousStep = wm.MoveToPreviousStep(node, "");

                // Send workflow e-mails
                if (SettingsKeyProvider.GetBoolValue(this.SiteName + ".CMSSendWorkflowEmails"))
                {
                    wm.SendWorkflowEmails(node, CMSContext.CurrentUser, originalStep, previousStep, WorkflowActionEnum.Rejected, "");
                }

                this.formElem.lblInfo.Text += " " + ResHelper.GetString("ContentEdit.WasRejected");
            }

            ReloadForm();

            RaiseOnAfterReject();
        }
    }


    /// <summary>
    /// CheckIn button click event handler.
    /// </summary>
    protected void btnCheckIn_Click(object sender, EventArgs e)
    {
        if (IsBannedIP())
        {
            return;
        }

        if (node != null)
        {
            // Validate the form first
            if (formElem.BasicForm.ValidateData())
            {
                // Save the document first
                if (this.SaveChanges || !confirmChanges)
                {
                    SaveDocument();
                }
                else
                {
                    this.formElem.BasicForm.LoadControlValues();
                }

                // Check in the document
                VersionManager verMan = new VersionManager(tree);

                // Check in the document        
                verMan.CheckIn(node, null, null);

                this.formElem.lblInfo.Text += " " + ResHelper.GetString("ContentEdit.WasCheckedIn");
            }

            ReloadForm();
        }
    }


    /// <summary>
    /// CheckOut button click event handler.
    /// </summary>
    protected void btnCheckOut_Click(object sender, EventArgs e)
    {
        if (IsBannedIP())
        {
            return;
        }

        // Check out the document
        if (node != null)
        {
            TreeProvider tree = new TreeProvider(CMSContext.CurrentUser);
            VersionManager verMan = new VersionManager(tree);

            verMan.EnsureVersion(node, node.IsPublished);

            // Check out the document
            verMan.CheckOut(node);
        }

        ReloadForm();
    }


    /// <summary>
    /// UndoCheckOut click event handler.
    /// </summary>
    protected void btnUndoCheckOut_Click(object sender, EventArgs e)
    {
        if (IsBannedIP())
        {
            return;
        }

        // Check out the document
        if (node != null)
        {
            TreeProvider tree = new TreeProvider(CMSContext.CurrentUser);
            VersionManager verMan = new VersionManager(tree);

            // Undo check out
            verMan.UndoCheckOut(node);
        }

        ReloadForm();
        this.formElem.LoadForm();

        // Reload the values if the form
        this.formElem.BasicForm.LoadControlValues();
    }


    /// <summary>
    /// New class selection click event handler.
    /// </summary>
    protected void btnClass_Command(object sender, CommandEventArgs e)
    {
        int newClassId = ValidationHelper.GetInteger(e.CommandArgument, 0);
        if (newClassId > 0)
        {
            this.ClassID = newClassId;
            this.ReloadData(true);
        }
    }


    /// <summary>
    /// OK button click event handler.
    /// </summary>
    protected void btnOK_Click(object sender, EventArgs e)
    {
        if (IsBannedIP())
        {
            return;
        }

        Action = "newculture";
        if (radCopy.Checked)
        {
            CopyDefaultDataFromDocumentID = ValidationHelper.GetInteger(this.lstCultures.SelectedValue, 0);
        }
        else
        {
            CopyDefaultDataFromDocumentID = 0;
        }

        this.ReloadData(true);
    }


    /// <summary>
    /// Yes button click event handler.
    /// </summary>
    protected void btnYes_Click(object sender, EventArgs e)
    {
        if (IsBannedIP())
        {
            return;
        }

        // Prepare the where condition
        string where = "NodeID = " + this.NodeID;

        // Get the documents
        TreeProvider tree = new TreeProvider(CMSContext.CurrentUser);
        DataSet ds = null;
        if (chkAllCultures.Checked)
        {
            ds = tree.SelectNodes(this.SiteName, "/%", TreeProvider.ALL_CULTURES, true, null, where, null, -1, false);
        }
        else
        {
            ds = tree.SelectNodes(this.SiteName, "/%", this.CultureCode, false, null, where, null, -1, false);
        }


        if (!DataHelper.DataSourceIsEmpty(ds))
        {
            // Get node alias
            string nodeAlias = ValidationHelper.GetString(DataHelper.GetDataRowValue(ds.Tables[0].Rows[0], "NodeAlias"), "");
            // Get parent alias path
            string parentAliasPath = TreePathUtils.GetParentPath(ValidationHelper.GetString(DataHelper.GetDataRowValue(ds.Tables[0].Rows[0], "NodeAliasPath"), ""));

            // Delete the documents
            foreach (DataRow dr in ds.Tables[0].Rows)
            {
                string aliasPath = ValidationHelper.GetString(dr["NodeAliasPath"], "");
                string culture = ValidationHelper.GetString(dr["DocumentCulture"], "");
                string className = ValidationHelper.GetString(dr["ClassName"], "");

                // Get the node
                TreeNode node = tree.SelectSingleNode(this.SiteName, aliasPath, culture, false, className, false);

                if (node != null)
                {
                    // Check delete permissions
                    bool hasUserDeletePermission = !this.CheckPermissions || IsUserAuthorizedToDeleteDocument(node, chkDestroy.Checked);

                    if (hasUserDeletePermission)
                    {
                        // Delete the document
                        try
                        {
                            int parentId = node.NodeParentID;
                            if (parentId <= 0)
                            {
                                parentId = node.NodeID;
                            }
                            if (!DocumentHelper.DeleteDocument(node, tree, this.chkAllCultures.Checked, this.chkDestroy.Checked, true))
                            {
                                parentId = node.NodeID;
                            }

                            this.Action = "edit";
                            this.ReloadData(true);
                        }
                        catch (Exception ex)
                        {
                            EventLogProvider log = new EventLogProvider(tree.Connection);
                            log.LogEvent("E", DateTime.Now, "Content", "DELETEDOC", CMSContext.CurrentUser.UserID, CMSContext.CurrentUser.UserName, node.NodeID, node.DocumentName, HTTPHelper.GetUserHostAddress(), ex.Message + ": " + ex.StackTrace, CMSContext.CurrentSite.SiteID, HTTPHelper.GetAbsoluteUri());
                            AddAlert(ResHelper.GetString("ContentRequest.DeleteFailed") + " : " + ex.Message);
                            return;
                        }
                    }
                    // Access denied - not authorized to delete the document
                    else
                    {
                        AddAlert(String.Format(ResHelper.GetString("cmsdesk.notauthorizedtodeletedocument"), node.NodeAliasPath));
                        return;
                    }
                }
                else
                {
                    AddAlert(ResHelper.GetString("ContentRequest.ErrorMissingSource"));
                    return;
                }
            }

            RaiseOnAfterDelete();

            string rawUrl = URLRewriter.RawUrl;
            if ((nodeAlias != "") && (rawUrl.Substring(rawUrl.LastIndexOf('/')).Contains(nodeAlias)))
            {
                // Redirect to the parent url when current url belongs to deleted document
                UrlHelper.Redirect(CMSContext.GetUrl(parentAliasPath));
            }
            else
            {
                // Redirect to current url
                UrlHelper.Redirect(rawUrl);
            }
        }
        else
        {
            AddAlert(ResHelper.GetString("DeleteDocument.CultureNotExists"));
            return;
        }
    }


    /// <summary>
    /// No button click event handler.
    /// </summary>
    protected void btnNo_Click(object sender, EventArgs e)
    {
        if (IsBannedIP())
        {
            return;
        }

        this.Action = "edit";
        this.ReloadData(false);
    }


    /// <summary>
    /// Delete button click event handler.
    /// </summary>
    protected void btnDelete_Click(object sender, EventArgs e)
    {
        if (IsBannedIP())
        {
            return;
        }

        this.Action = "delete";
        this.ReloadData(true);
    }


    protected override void OnPreRender(EventArgs e)
    {
        // Render the styles link if live site mode
        if (this.Visible && (CMSContext.ViewMode == ViewModeEnum.LiveSite))
        {
            string tag = "";
            if ((this.Page.Theme != "") && File.Exists(Server.MapPath("~/App_Themes/" + this.Page.Theme + "/DesignMode.css")))
            {
                tag = HTMLHelper.GetCSSFileLink(ResolveUrl("~/App_Themes/" + this.Page.Theme + "/DesignMode.css"));
            }
            else
            {
                tag = HTMLHelper.GetCSSFileLink(ResolveUrl("~/App_Themes/Default/DesignMode.css"));
            }

            this.Page.Header.Controls.Add(new LiteralControl(tag));
        }

        base.OnPreRender(e);
    }


    /// <summary>
    /// Checks whether the user is authorized to delete document.
    /// </summary>
    /// <param name="node">Document node</param>
    /// <param name="destroyDocHistory">Delete document history?</param>
    protected bool IsUserAuthorizedToDeleteDocument(TreeNode node, bool deleteDocHistory)
    {
        bool isAuthorized = true;

        CurrentUserInfo currentUser = CMSContext.CurrentUser;

        // Check delete permission
        if (currentUser.IsAuthorizedPerDocument(node, NodePermissionsEnum.Delete) == AuthorizationResultEnum.Allowed)
        {
            if (deleteDocHistory)
            {
                // Check destroy permisson
                if (!(currentUser.IsAuthorizedPerDocument(node, NodePermissionsEnum.Destroy) == AuthorizationResultEnum.Allowed))
                {
                    isAuthorized = false;
                }
            }
        }
        else
        {
            isAuthorized = false;
        }

        return isAuthorized;
    }


    /// <summary>
    /// Check if user IP is banned.
    /// </summary>
    private bool IsBannedIP()
    {
        // Check banned ip
        if (!BannedIPInfoProvider.IsAllowed(CMSContext.CurrentSite.SiteID, BanControlEnum.AllNonComplete))
        {
            lblError.Visible = true;
            lblError.Text = ResHelper.GetString("General.BannedIP");
            return true;
        }
        return false;
    }


    /// <summary>
    /// Returns alternative form name in full version - 'ClassName.AltFormCodeName'
    /// </summary>
    /// <param name="className">Class name</param>
    private string GetAltFormFullName(string className)
    {
        if (!string.IsNullOrEmpty(this.AlternativeFormName) && !string.IsNullOrEmpty(className) && !this.AlternativeFormName.StartsWith(className))
        {
            if (this.AlternativeFormName.Contains("."))
            {
                // Remove class name if it is different from class name in parameter
                this.AlternativeFormName = this.AlternativeFormName.Remove(0, this.AlternativeFormName.LastIndexOf(".") + 1);
            }
            return className + "." + this.AlternativeFormName;
        }
        else
        {
            return this.AlternativeFormName;
        }
    }


    /// <summary>
    /// Raises the OnAfterApprove event
    /// </summary>
    private void RaiseOnAfterApprove()
    {
        if (OnAfterApprove != null)
        {
            OnAfterApprove(this, null);
        }
    }


    /// <summary>
    /// Raises the OnAfterReject event
    /// </summary>
    private void RaiseOnAfterReject()
    {
        if (OnAfterReject != null)
        {
            OnAfterReject(this, null);
        }
    }


    /// <summary>
    /// Raises the OnAfterDelete event
    /// </summary>
    private void RaiseOnAfterDelete()
    {
        if (OnAfterDelete != null)
        {
            OnAfterDelete(this, null);
        }
    }

    #endregion
}
