using System;
using System.Collections;

using CMS.Community;
using CMS.GlobalHelper;
using CMS.LicenseProvider;
using CMS.SettingsProvider;
using CMS.SiteProvider;
using CMS.UIControls;
using CMS.CMSHelper;

public partial class CMSModules_Friends_Approve : CMSModalPage
{
    #region "Variables"

    protected int userId = 0;
    protected CurrentUserInfo currentUser = null;

    #endregion


    protected void Page_Load(object sender, EventArgs e)
    {
        userId = QueryHelper.GetInteger("userid", 0);
        currentUser = CMSContext.CurrentUser;

        // Check license
        if (DataHelper.GetNotEmpty(UrlHelper.GetCurrentDomain(), string.Empty) != string.Empty)
        {
            LicenseHelper.CheckFeatureAndRedirect(UrlHelper.GetCurrentDomain(), FeatureEnum.Friends);
        }

        FriendsApprove.SelectedFriends = null;
        FriendsApprove.OnCheckPermissions += new CMSAdminControl.CheckPermissionsEventHandler(FriendsApprove_OnCheckPermissions);

        int requestedId = QueryHelper.GetInteger("requestid", 0);
        int friendshipId = 0;
        CurrentMaster.Title.TitleText = ResHelper.GetString("friends.approvefriendship");
        CurrentMaster.Title.TitleImage = "~/App_Themes/" + Page.Theme + "/Images/CMSModules/Friends/waitingforapproval.gif";

        // Multiple selection
        if (Request["ids"] != null)
        {
            string[] items = Request["ids"].Split(',');
            if (items.Length > 0)
            {
                ArrayList friends = new ArrayList();
                foreach (string item in items)
                {
                    friends.Add(ValidationHelper.GetInteger(item, 0));
                }
                FriendsApprove.SelectedFriends = friends;
                if (friends.Count == 1)
                {
                    friendshipId = Convert.ToInt32(friends[0]);
                }
            }
        }
        // For one user
        else
        {
            FriendsApprove.RequestedUserID = requestedId;
        }

        FriendInfo fi = null;
        if (friendshipId != 0)
        {
            fi = FriendInfoProvider.GetFriendInfo(friendshipId);
        }
        else if (requestedId != 0)
        {
            fi = FriendInfoProvider.GetFriendInfo(userId, requestedId);
        }

        if (fi != null)
        {
            UserInfo requestedUser = (userId == fi.FriendRequestedUserID) ? UserInfoProvider.GetFullUserInfo(fi.FriendUserID) : UserInfoProvider.GetFullUserInfo(fi.FriendRequestedUserID);
            string fullUserName = UserInfoProvider.GetFullUserName(requestedUser.UserName, requestedUser.UserNickName);
            Page.Title = ResHelper.GetString("friends.approvefriendshipwith") + " " + HTMLHelper.HTMLEncode(fullUserName);
            CurrentMaster.Title.TitleText = Page.Title;
        }

        // Set current user
        FriendsApprove.UserID = userId;
    }

    void FriendsApprove_OnCheckPermissions(string permissionType, CMSAdminControl sender)
    {
        // Approve for another user
        if (!currentUser.IsAuthorizedPerResource("CMS.Friends", permissionType) && (currentUser.UserID != userId))
        {
            RedirectToAccessDenied("CMS.Friends", permissionType);
        }
    }
}
