using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Text.RegularExpressions;

using CMS.CMSHelper;
using CMS.SettingsProvider;
using CMS.GlobalHelper;
using CMS.UIControls;
using CMS.PortalEngine;
using CMS.SiteProvider;

public partial class CMSPages_PortalTemplate : PortalPage
{
    /// <summary>
    /// Gets the title for current page (hotfix for 4.0)
    /// </summary>    
    private string GetTitle()
    {
        string result = null;

        PageInfo currentPage = CMSContext.CurrentPageInfo;
        if (currentPage != null)
        {
            SiteInfo si = SiteInfoProvider.GetSiteInfo(currentPage.NodeSiteId);

            // Get default site Title
            string prefix = SettingsKeyProvider.GetStringValue(si.SiteName + ".CMSPageTitlePrefix");
            string format = SettingsKeyProvider.GetStringValue(si.SiteName + ".CMSPageTitleFormat");

            string titleOrName = DataHelper.GetNotEmpty(currentPage.DocumentPageTitle, currentPage.DocumentName);

            // Resolve special values
            result = Regex.Replace(format, "{%prefix%}", prefix, RegexOptions.IgnoreCase);
            result = Regex.Replace(result, "{%pagetitle_orelse_name%}", titleOrName, RegexOptions.IgnoreCase);
            
            result = CMSContext.CurrentResolver.ResolveMacros(result);
        }

        return result;
    }


    protected override void OnPreInit(EventArgs e)
    {
        base.OnPreInit(e);

        CMSContext.CurrentTitle = GetTitle();

        // Init the page components
        this.PageManager = this.manPortal;
        this.manPortal.SetMainPagePlaceholder(this.plcRoot);
        this.plcRoot.CacheMinutes = CacheHelper.CachePageInfoMinutes(CMSContext.CurrentSiteName);

        this.ManagersContainer = this.plcManagers;
        this.LogsContainer = this.plcLogs;

        this.ScriptManagerControl = this.manScript;
    }


    protected override void OnPreRender(EventArgs e)
    {
        base.OnPreRender(e);

        // Init the header tags
        this.ltlTags.Text = this.HeaderTags;
    }
}
